@@ -97,6 +97,7 @@ struct audio_adapter {
 	GIOChannel *hsp_hs_server;
 };
 
+static int max_connected_headsets = 1;
 static DBusConnection *connection = NULL;
 static GKeyFile *config = NULL;
 static GSList *adapters = NULL;
@@ -452,8 +453,16 @@ static void ag_io_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 	if (!device)
 		goto drop;
 
-	if (!device->headset)
+	if (!manager_allow_headset_connection(&device->src)) {
+		debug("Refusing headset: too many existing connections");
+		goto drop;
+	}
+
+	if (!device->headset) {
 		btd_device_add_uuid(device->btd_dev, remote_uuid);
+		if (!device->headset)
+			goto drop;
+	}
 
 	if (headset_get_state(device) > HEADSET_STATE_DISCONNECTED) {
 		debug("Refusing new connection since one already exists");
@@ -506,8 +515,7 @@ static int headset_server_init(struct audio_adapter *adapter)
 						&err);
 		if (err) {
 			debug("audio.conf: %s", err->message);
-			g_error_free(err);
-			err = NULL;
+			g_clear_error(&err);
 		} else
 			master = tmp;
 	}
@@ -591,8 +599,7 @@ static int gateway_server_init(struct audio_adapter *adapter)
 						&err);
 		if (err) {
 			debug("audio.conf: %s", err->message);
-			g_error_free(err);
-			err = NULL;
+			g_clear_error(&err);
 		} else
 			master = tmp;
 	}
@@ -914,11 +921,19 @@ int audio_manager_init(DBusConnection *conn, GKeyFile *conf)
 					&err);
 	if (err) {
 		debug("audio.conf: %s", err->message);
-		g_error_free(err);
-		err = NULL;
+		g_clear_error(&err);
 	} else
 		enabled.hfp = b;
 
+	err = NULL;
+	i = g_key_file_get_integer(config, "Headset", "MaxConnected",
+					&err);
+	if (err) {
+		debug("audio.conf: %s", err->message);
+		g_clear_error(&err);
+	} else
+		max_connected_headsets = i;
+
 proceed:
 	if (enabled.headset) {
 		telephony_init();
@@ -1062,3 +1077,28 @@ struct audio_device *manager_get_device(const bdaddr_t *src,
 
 	return dev;
 }
+
+gboolean manager_allow_headset_connection(bdaddr_t *src)
+{
+	GSList *l;
+	int connected = 0;
+
+	for (l = devices; l != NULL; l = l->next) {
+		struct audio_device *dev = l->data;
+		struct headset *hs = dev->headset;
+
+		if (bacmp(&dev->src, src))
+			continue;
+
+		if (!hs)
+			continue;
+
+		if (headset_get_state(dev) > HEADSET_STATE_DISCONNECTED)
+			connected++;
+
+		if (connected >= max_connected_headsets)
+			return FALSE;
+	}
+
+	return TRUE;
+}
