@@ -92,6 +92,13 @@
 #define OPERATOR_MODE_DEREGISTER	2
 #define OPERATOR_MODE_MANUAL_AUTO	4
 
+/* Some common number types */
+#define NUMBER_TYPE_UNKNOWN		128
+#define NUMBER_TYPE_TELEPHONY		129
+#define NUMBER_TYPE_INTERNATIONAL	145
+#define NUMBER_TYPE_NATIONAL		161
+#define NUMBER_TYPE_VOIP		255
+
 /* Extended Audio Gateway Error Result Codes */
 typedef enum {
 	CME_ERROR_NONE			= -1,
@@ -116,6 +123,7 @@ typedef enum {
 	CME_ERROR_DIAL_STRING_TOO_LONG	= 26,
 	CME_ERROR_INVALID_DIAL_STRING	= 27,
 	CME_ERROR_NO_NETWORK_SERVICE	= 30,
+	CME_ERROR_NETWORK_TIMEOUT	= 31,
 	CME_ERROR_NETWORK_NOT_ALLOWED	= 32,
 } cme_error_t;
 
@@ -144,6 +152,8 @@ void telephony_transmit_dtmf_req(void *telephony_device, char tone);
 void telephony_subscriber_number_req(void *telephony_device);
 void telephony_list_current_calls_req(void *telephony_device);
 void telephony_operator_selection_req(void *telephony_device);
+void telephony_call_hold_req(void *telephony_device, const char *cmd);
+void telephony_disable_nr_and_ec_req(void *telephony_device, gboolean enable);
 
 /* AG responses to HF requests. These are implemented by headset.c */
 int telephony_event_reporting_rsp(void *telephony_device, cme_error_t err);
@@ -156,6 +166,8 @@ int telephony_transmit_dtmf_rsp(void *telephony_device, cme_error_t err);
 int telephony_subscriber_number_rsp(void *telephony_device, cme_error_t err);
 int telephony_list_current_calls_rsp(void *telephony_device, cme_error_t err);
 int telephony_operator_selection_rsp(void *telephony_device, cme_error_t err);
+int telephony_call_hold_rsp(void *telephony_device, cme_error_t err);
+int telephony_disable_nr_and_ec_rsp(void *telephony_device, cme_error_t err);
 
 /* Event indications by AG. These are implemented by headset.c */
 int telephony_event_ind(int index);
@@ -163,7 +175,7 @@ int telephony_response_and_hold_ind(int rh);
 int telephony_incoming_call_ind(const char *number, int type);
 int telephony_calling_stopped_ind(void);
 int telephony_ready_ind(uint32_t features, const struct indicator *indicators,
-			int rh);
+			int rh, const char *chld);
 int telephony_list_current_call_ind(int idx, int dir, int status, int mode,
 					int mprty, const char *number,
 					int type);
