@@ -40,6 +40,7 @@
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/rfcomm.h>
 #include <bluetooth/sdp.h>
+#include <bluetooth/sdp_lib.h>
 
 #include <glib.h>
 #include <gdbus.h>
@@ -72,9 +73,9 @@ struct serial_port {
 	DBusMessage	*msg;		/* for name listener handling */
 	int16_t		id;		/* RFCOMM device id */
 	uint8_t		channel;	/* RFCOMM channel */
-	char		*name;		/* service friendly name */
 	char		*uuid;		/* service identification */
 	char		*dev;		/* RFCOMM device name */
+	int		fd;		/* Opened file descriptor */
 	guint		listener_id;
 	struct serial_device *device;
 };
@@ -101,15 +102,25 @@ static struct serial_port *find_port(GSList *ports, const char *pattern)
 
 	for (l = ports; l != NULL; l = l->next) {
 		struct serial_port *port = l->data;
-
-		if (!strcasecmp(port->name, pattern))
-			return port;
+		char *uuid_str;
+		int ret;
 
 		if (!strcasecmp(port->uuid, pattern))
 			return port;
 
 		if (port->dev && !strcmp(port->dev, pattern))
 			return port;
+
+		uuid_str = bt_name2string(pattern);
+		if (!uuid_str)
+			continue;
+
+		ret = strcasecmp(port->uuid, uuid_str);
+
+		g_free(uuid_str);
+
+		if (ret == 0)
+			return port;
 	}
 
 	return NULL;
@@ -127,6 +138,11 @@ static int port_release(struct serial_port *port)
 	if (rfcomm_ctl < 0)
 		return -errno;
 
+	if (port->fd >= 0) {
+		close(port->fd);
+		port->fd = -1;
+	}
+
 	memset(&req, 0, sizeof(req));
 	req.dev_id = port->id;
 
@@ -145,16 +161,15 @@ static int port_release(struct serial_port *port)
 
 	g_free(port->dev);
 	port->dev = NULL;
-	port->id = 0;
+	port->id = -1;
 	close(rfcomm_ctl);
 	return -err;
 }
 
 static void serial_port_free(struct serial_port *port)
 {
-	if (port->id)
+	if (port->id >= 0)
 		port_release(port);
-	g_free(port->name);
 	g_free(port->uuid);
 	g_free(port);
 }
@@ -171,7 +186,7 @@ static void port_owner_exited(DBusConnection *conn, void *user_data)
 {
 	struct serial_port *port = user_data;
 
-	if (port->id)
+	if (port->id >= 0)
 		port_release(port);
 
 	port->listener_id = 0;
@@ -224,6 +239,7 @@ static void open_notify(int fd, int err, struct serial_port *port)
 		port_release(port);
 		reply = failed(port->msg, strerror(err));
 	} else {
+		port->fd = fd;
 		reply = g_dbus_create_reply(port->msg,
 				DBUS_TYPE_STRING, &port->dev,
 				DBUS_TYPE_INVALID);
@@ -370,7 +386,7 @@ static DBusMessage *port_disconnect(DBusConnection *conn,
 {
 	struct serial_device *device = user_data;
 	struct serial_port *port;
-	const char *dev;
+	const char *dev, *owner, *caller;
 
 	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &dev,
 						DBUS_TYPE_INVALID) == FALSE)
@@ -383,7 +399,12 @@ static DBusMessage *port_disconnect(DBusConnection *conn,
 	if (!port->listener_id)
 		return failed(msg, "Not connected");
 
-	if (port->id)
+	owner = dbus_message_get_sender(port->msg);
+	caller = dbus_message_get_sender(msg);
+	if (!g_str_equal(owner, caller))
+		return failed(msg, "Operation not permited");
+
+	if (port->id >= 0)
 		port_release(port);
 
 	g_dbus_remove_watch(conn, port->listener_id);
@@ -427,8 +448,7 @@ static struct serial_device *create_serial_device(DBusConnection *conn,
 }
 
 int port_register(DBusConnection *conn, const char *path, bdaddr_t *src,
-		  bdaddr_t *dst, const char *name, const char *uuid,
-		  uint8_t channel)
+		  bdaddr_t *dst, const char *uuid, uint8_t channel)
 {
 	struct serial_device *device;
 	struct serial_port *port;
@@ -445,33 +465,27 @@ int port_register(DBusConnection *conn, const char *path, bdaddr_t *src,
 		return 0;
 
 	port = g_new0(struct serial_port, 1);
-	port->name = g_strdup(name);
 	port->uuid = g_strdup(uuid);
 	port->channel = channel;
 	port->device = device;
+	port->id = -1;
+	port->fd = -1;
 
 	device->ports = g_slist_append(device->ports, port);
 
 	return 0;
 }
 
-int port_unregister(const char *path, const char *uuid)
+int port_unregister(const char *path)
 {
 	struct serial_device *device;
-	struct serial_port *port;
 
 	device = find_device(devices, path);
 	if (!device)
 		return -ENOENT;
 
-	port = find_port(device->ports, uuid);
-	if (!port)
-		return -ENOENT;
-
-	device->ports = g_slist_remove(device->ports, port);
-	serial_port_free(port);
-	if (device->ports)
-		return 0;
+	g_slist_foreach(device->ports, (GFunc) serial_port_free, NULL);
+	g_slist_free(device->ports);
 
 	g_dbus_unregister_interface(device->conn, path, SERIAL_PORT_INTERFACE);
 
