@@ -186,11 +186,7 @@ struct agent *agent_create(struct btd_adapter *adapter, const char *name,
 				const char *path, uint8_t capability,
 				agent_remove_cb cb, void *remove_cb_data)
 {
-	struct agent *agent, *adapter_agent;
-
-	adapter_agent = adapter_get_agent(adapter);
-	if (adapter_agent && g_str_equal(adapter_agent->name, name))
-		return NULL;
+	struct agent *agent;
 
 	agent = g_new0(struct agent, 1);
 
